Package: methylCC
Title: Estimate the cell composition of whole blood in DNA methylation
        samples
Version: 1.24.0
Authors@R: c(
    person(c("Stephanie", "C."), "Hicks", role = c("aut", "cre"), 
		email = "shicks19@jhu.edu", 
		comment = c(ORCID = "0000-0002-7858-0231")),
    person("Rafael", "Irizarry", role = "aut", 
		comment = c(ORCID = "0000-0002-3944-4309")))
Imports: Biobase, GenomicRanges, IRanges, S4Vectors, dplyr, magrittr,
        minfi, bsseq, quadprog, plyranges, stats, utils, bumphunter,
        genefilter, methods, IlluminaHumanMethylation450kmanifest,
        IlluminaHumanMethylation450kanno.ilmn12.hg19
Depends: R (>= 3.6), FlowSorted.Blood.450k
Suggests: rmarkdown, knitr, testthat (>= 2.1.0), BiocGenerics,
        BiocStyle, tidyr, ggplot2
Description: A tool to estimate the cell composition
    of DNA methylation whole blood sample measured on any
    platform technology (microarray and sequencing).
biocViews: Microarray, Sequencing, DNAMethylation, MethylationArray,
        MethylSeq, WholeGenome
VignetteBuilder: knitr
RoxygenNote: 6.1.1
Encoding: UTF-8
License: GPL-3
BugReports: https://github.com/stephaniehicks/methylCC/
URL: https://github.com/stephaniehicks/methylCC/
git_url: https://git.bioconductor.org/packages/methylCC
git_branch: RELEASE_3_22
git_last_commit: 98b2564
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:02:06 UTC; biocbuild
Author: Stephanie C. Hicks [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7858-0231>),
  Rafael Irizarry [aut] (ORCID: <https://orcid.org/0000-0002-3944-4309>)
Maintainer: Stephanie C. Hicks <shicks19@jhu.edu>

# Generated by roxygen2: do not edit by hand

export(estimatecc)
exportClasses(estimatecc)
exportMethods(cell_counts)
import(FlowSorted.Blood.450k)
import(GenomicRanges)
import(IlluminaHumanMethylation450kanno.ilmn12.hg19)
import(IlluminaHumanMethylation450kmanifest)
import(methods)
import(minfi)
importFrom(Biobase,pData)
importFrom(Biobase,sampleNames)
importFrom(IRanges,subsetByOverlaps)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(bsseq,BSseq)
importFrom(bsseq,getBSseq)
importFrom(bumphunter,bumphunter)
importFrom(bumphunter,clusterMaker)
importFrom(bumphunter,loessByCluster)
importFrom(dplyr,funs)
importFrom(dplyr,group_by)
importFrom(dplyr,summarise_all)
importFrom(genefilter,rowttests)
importFrom(magrittr,"%>%")
importFrom(plyranges,arrange)
importFrom(quadprog,solve.QP)
importFrom(stats,model.matrix)
importFrom(stats,runif)
importFrom(stats,var)
importFrom(utils,data)
importFrom(utils,head)

# methylCC

### Citation 

Hicks SC, Irizarry RA. (2019). _Genome Biology_ **20**, 261. [https://doi.org/10.1186/s13059-019-1827-8](https://doi.org/10.1186/s13059-019-1827-8)

### Why use methylCC? 

This is a package to estimate the cell composition 
    of whole blood in DNA methylation measured on any 
    platform technology (e.g. Illumina 450K microarray, 
    whole genome bisulfite sequencing (WGBS) and 
    reduced representation bisulfite sequencing (RRBS)). 

For help with the **methylCC** R-package, there is a vignette available in the /vignettes folder.
  
### Installing methylCC

The R-package **methylCC** can be installed from Github using the R 
package [devtools](https://github.com/hadley/devtools): 


Use  to install the latest version of **methylCC** from Github:
```s
library(devtools)
install_github("stephaniehicks/methylCC")
```

It can also be installed using Bioconductor: 

```s
# install BiocManager from CRAN (if not already installed)
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")

# install methylCC package
BiocManager::install("methylCC")
```

After installation, the package can be loaded into R.
```s
library(methylCC)
```

# Bug reports
Report bugs as issues on the [GitHub repository](https://github.com/stephaniehicks/methylCC)


# Authors

* [Stephanie C. Hicks](https://github.com/stephaniehicks)
* [Rafael A. Irizarry](https://github.com/ririzarr)

#' @title .splitit
#' 
#' @description helper function to split along a variable
#' 
#' @return A list to be used in \code{find_dmrs()}
#' 
#' @param x a vector
#' 
.splitit <- function(x) {
  split(seq(along = x), x)
}

#' @title Pick target positions
#'
#' @description Pick probes from target data using 
#' the indices in \code{dmp_regions}
#'
#' @param target_granges add more here.
#' @param target_object an optional argument which 
#' contains the meta-data for \code{target_granges}. If
#' \code{target_granges} already contains the meta-data, do not 
#' need to supply \code{target_object}.
#' @param target_cvg coverage reads for the target object
#' @param dmp_regions differentially methylated regions
#'
#' @return A list of GRanges objects to be used in 
#' \code{.preprocess_estimatecc()}
#'
#' @import GenomicRanges
#' @importFrom IRanges subsetByOverlaps 
#' @importFrom S4Vectors subjectHits queryHits
#' @importFrom magrittr %>%
#' @importFrom dplyr group_by summarise_all funs
#'
.pick_target_positions <- function(target_granges, target_object = NULL,
                                   target_cvg = NULL, dmp_regions)
{
  
  if(is(dmp_regions, "GRanges")){
    train_granges = dmp_regions
  } else {
    train_granges = makeGRangesFromDataFrame(dmp_regions, 
                                             keep.extra.columns=TRUE)
  }
  
  # Zmat regions that overlap with target data
  train_granges <- subsetByOverlaps(train_granges, target_granges) 
  
  # regions from Zmat (CpG level)
  subject_mtch <- subjectHits(findOverlaps(target_granges, train_granges)) 
  
  # CpGs from target data (CpG level)
  query_mtch <- queryHits(findOverlaps(target_granges, train_granges)) 
  
  # (CpG level)
  intersect_granges <- target_granges[query_mtch, ] 
  if(!is.null(target_object)){
    mcols(intersect_granges) <- target_object[query_mtch, ]
  }
  
  mcols(intersect_granges)$subject_mtch <- subject_mtch
  mcols(intersect_granges)$query_mtch <- query_mtch
  metadata = as.data.frame(mcols(intersect_granges)) 
  
  if(!is.null(target_cvg)){
    intersect_cvg <- target_granges[query_mtch, ]
    mcols(intersect_cvg) <- target_cvg[query_mtch, ]
    mcols(intersect_cvg)$subject_mtch <- subject_mtch
    mcols(intersect_cvg)$query_mtch <- query_mtch
    metadata_cvg = as.data.frame(mcols(intersect_cvg))
    
    # Sum M values and cvg values for all CpGs in each region (subject)
    . <- NULL 
    out_m = metadata %>% group_by(subject_mtch) %>% 
              summarise_all(funs(sum(.,na.rm = TRUE)))
    out_cvg = metadata_cvg %>% group_by(subject_mtch) %>% 
              summarise_all(funs(sum(.,na.rm = TRUE)))
    dmp_granges <- train_granges
    mcols(dmp_granges) <- 
      out_m[, !(colnames(out_m) %in% c("query_mtch", "subject_mtch"))] /
      out_cvg[, !(colnames(out_cvg) %in% c("query_mtch", "subject_mtch"))]
    mcols(intersect_granges) <- 
      metadata[, !(colnames(metadata) %in% c("query_mtch", "subject_mtch"))] /
      metadata_cvg[, !(colnames(metadata_cvg) %in% c("query_mtch", 
                                                     "subject_mtch"))]
  } else {
    # Average M values for all CpGs in each region (subject)
    out = metadata %>% group_by(subject_mtch) %>% 
              summarise_all(funs(mean(., na.rm = TRUE)))
    dmp_granges <- train_granges
    mcols(dmp_granges) <- out[, !(colnames(out) %in%
                                    c("query_mtch", "subject_mtch"))]
    mcols(intersect_granges) <- 
      metadata[, !(colnames(metadata) %in% c("query_mtch", "subject_mtch"))]
    
  }
  
  list("probeGRanges" = intersect_granges,
       "dmp_granges" = dmp_granges, "avg_dmr_dat" = mcols(dmp_granges))
}

#' @title Extract raw data
#'
#' @description Extract the methylation values and 
#' GRanges objects 
#'
#' @param object an object can be a \code{RGChannelSet}, 
#' \code{GenomicMethylSet} or \code{BSseq} object
#' 
#' @return A list preprocessed objects from the 
#' \code{RGChannelSet}, \code{GenomicMethylSet} or 
#' \code{BSseq} objects to be used in \code{.preprocess_estimatecc()}.
#'
#' @import minfi
#' @import IlluminaHumanMethylation450kmanifest
#' @import IlluminaHumanMethylation450kanno.ilmn12.hg19
#' @import GenomicRanges
#' @importFrom Biobase pData sampleNames
#' @importFrom bsseq BSseq getBSseq
.extract_raw_data <- function(object)
{
  if(is(object, "RGChannelSet")){
    Mset <- preprocessIllumina(object)
    Mset <- mapToGenome(Mset, mergeManifest = FALSE)
    pd = pData(Mset)
    gr_object <- granges(Mset)
    beta_values = getBeta(Mset, type = "Illumina")
    rm(Mset)
    output <- list(pd = pd, gr_object = gr_object, 
                   beta_values = beta_values)
  }
  if(is(object, "GenomicMethylSet")){
    pd = pData(object)
    gr_object <- granges(object)
    beta_values = getBeta(object, type = "Illumina")
    output <- list(pd = pd, gr_object = gr_object, 
                   beta_values = beta_values)
  }
  if(is(object, "BSseq")){
    cvg_targetbs <- getBSseq(object, type = "Cov")
    M_targetbs <- getBSseq(object, type = "M")
    gr_object <- granges(object)
    output <- list(cvg_targetbs = cvg_targetbs, 
                   M_targetbs = M_targetbs, gr_object = gr_object)
  }
  return(output)
}


#' @title .preprocess_estimatecc
#'
#' @description This function preprocesses the data 
#' before the \code{estimatecc()} function
#'
#' @param object an object can be a \code{RGChannelSet}, 
#' \code{GenomicMethylSet} or \code{BSseq} object
#' @param verbose TRUE/FALSE argument specifying if verbose
#' messages should be returned or not. Default is TRUE.
#' @param init_param_method method to initialize parameter estimates.
#' Choose between "random" (randomly sample) or "known_regions"
#' (uses unmethyalted and methylated regions that were identified
#' based on Reinus et al. (2012) cell sorted data.).
#' Defaults to "random".
#' @param celltype_specific_dmrs cell type specific differentially 
#' methylated regions (DMRs).
#' 
#' @return A list of object to be used in estimatecc
#'
#' @import minfi
#' @import IlluminaHumanMethylation450kmanifest
#' @import IlluminaHumanMethylation450kanno.ilmn12.hg19
#' @import GenomicRanges
#' @importFrom IRanges subsetByOverlaps 
#' @importFrom stats var
#' @importFrom utils data
#' @importFrom Biobase pData sampleNames
#' @importFrom bsseq BSseq getBSseq
#' 
# dat <- preprocess_estimatecc(BS, init_param_method = "random")
.preprocess_estimatecc <- function(object, verbose=TRUE,
                              init_param_method = "random", 
                              celltype_specific_dmrs = celltype_specific_dmrs)
{

  if(init_param_method == "known_regions"){
    offMethRegions = onMethRegions <- NULL 
    dir <- system.file("data", package="methylCC")
    file_off <- file.path(dir, "offMethRegions.RData") 
    file_on  <- file.path(dir, "onMethRegions.RData") 
    if(file.exists(file_off)){
      load(file = file_off)
      grd0 = makeGRangesFromDataFrame(offMethRegions)
    } 
    if(file.exists(file_on)){
      load(file = file_on)
      grd1 = makeGRangesFromDataFrame(onMethRegions)
    } 
  }

  if(is(object, "RGChannelSet") || is(object, "GenomicMethylSet")) {       
    if(verbose){
      message("[estimatecc] Preprocessing RGChannelSet or
              GenomicMethylSet object.") 
    }
      eout <- .extract_raw_data(object) 
      keep_dmrs <- 
        .pick_target_positions(target_granges=eout$gr_object,
                               target_object = eout$beta_values,
                               dmp_regions=celltype_specific_dmrs)$dmp_granges
      zmat_sub <- subsetByOverlaps(celltype_specific_dmrs, 
                                   keep_dmrs, type = "equal")
      
      if(init_param_method == "known_regions"){
        y_d0 <- as.data.frame(
                  .pick_target_positions(target_granges = eout$gr_object,
                                         target_object = eout$beta_values,
                                         dmp_regions = grd0)$avg_dmr_dat)
        a0init <- mean(colMeans(y_d0))
        sig0init <- mean(apply(y_d0, 2, var))
        y_d1 <- as.data.frame(
                  .pick_target_positions(target_granges = eout$gr_object,
                                         target_object = eout$beta_values,
                                         dmp_regions = grd1)$avg_dmr_dat)
        a1init <- mean(colMeans(y_d1))
        sig1init <- mean(apply(y_d1, 2, var))
      }
  }
  
  if(is(object, "BSseq")){
      if(verbose){
        message("[estimatecc] Extracting BSSeq data.") }
      eout <- .extract_raw_data(object) 

      keep_dmrs <- .pick_target_positions(target_granges = eout$gr_object,
                                         target_object = eout$M_targetbs, 
                                         target_cvg = eout$cvg_targetbs,
                              dmp_regions = celltype_specific_dmrs)$dmp_granges
      zmat_sub <- subsetByOverlaps(celltype_specific_dmrs, keep_dmrs,
                                   type = "equal")
      if(verbose){
        mes <- "[estimatecc] BSseq object contained
                %s out of %s celltype-specific regions."
        message(sprintf(mes, length(keep_dmrs),
                        length(celltype_specific_dmrs)))
      }
      
      if(init_param_method == "known_regions"){
        y_d0 <- as.data.frame(
          .pick_target_positions(
            target_granges = eout$gr_object,
            target_object = eout$M_targetbs,
            target_cvg = eout$cvg_targetbs,
            dmp_regions = grd0)$avg_dmr_dat)
        a0init <- mean(colMeans(y_d0))
        sig0init <- mean(apply(y_d0, 2, var))
        y_d1 <- as.data.frame(
          .pick_target_positions(
            target_granges = eout$gr_object,
            target_object = eout$M_targetbs,
            target_cvg = eout$cvg_targetbs,
            dmp_regions = grd1)$avg_dmr_dat)
        a1init <- mean(colMeans(y_d1))
        sig1init <- mean(apply(y_d1, 2, var))
      }
    }
  
  ymat <- mcols(keep_dmrs) 
  if(!is.null(sampleNames(object))){ 
    colnames(ymat) <- sampleNames(object)
  }
  n <- ncol(mcols(keep_dmrs))
  ids <- colnames(mcols(zmat_sub))
  zmat <- as.matrix(as.data.frame(mcols(zmat_sub)))

  if(init_param_method == "known_regions"){
    output <- list(ymat = ymat, n = n, ids = ids, zmat = zmat,
         a0init = a0init, a1init = a1init,
         sig0init = sig0init, sig1init = sig1init,
         gr_object = eout$gr_object, keep_dmrs = keep_dmrs)
    rm(eout)
  } else {
    output <- list(ymat = ymat, n = n, ids = ids, zmat = zmat,
         gr_object = eout$gr_object, keep_dmrs = keep_dmrs)
    rm(eout)
  }
  return(output)
}


#' @title .initialize_theta
#'
#' @description Creates a container with initial theta
#' parameter estimates
#'
#' @param n Number of samples
#' @param K Number of cell types
#' @param alpha0 Default NULL. Initial mean methylation 
#' level in unmethylated regions
#' @param alpha1 Default NULL. Initial mean methylation
#'  level in methylated regions
#' @param sig0 Default NULL. Initial var methylation 
#' level in unmethylated regions
#' @param sig1 Default NULL. Initial var methylation 
#' level in methylated regions
#' @param tau Default NULL. Initial var for measurement error
#' 
#' @return A data frame with initial parameter estimates
#' to be used in \code{.initializeMLEs()}.
#' 
#' @importFrom stats runif
#'
.initialize_theta <- function(n, K, alpha0 = NULL, alpha1 = NULL,
                              sig0 = NULL, sig1 = NULL, tau = NULL){
  if(is.null(alpha0)){ alpha0 <- runif(1, 0.10, 0.3) }
  if(is.null(alpha1)){ alpha1 <- runif(1, 0.70, 0.9) }
  if(is.null(sig0)){ sig0 <- runif(1, 0.01, 0.05)^2 }
  if(is.null(sig1)){ sig1 <- runif(1, 0.01, 0.05)^2 }
  if(is.null(tau)){ tau <- runif(1, 0.001, 0.01)^2 }
  return(data.frame("alpha0" = alpha0, "alpha1" = alpha1,
                    "sig0" = sig0, "sig1" = sig1, "tau" = tau,
                    "lik_fun" = 0))
}

#' @title .initializeMLEs 
#' 
#' @description Helper functions to initialize 
#' MLEs in \code{estimatecc()}. 
#' 
#' @param init_param_method method to initialize parameter estimates.
#' Choose between "random" (randomly sample) or "known_regions"
#' (uses unmethyalted and methylated regions that were identified
#' based on Reinus et al. (2012) cell sorted data.).
#' Defaults to "random".
#' @param n Number of samples
#' @param K Number of cell types
#' @param Ys observed methylation levels in samples 
#' provided by user of dimension R x n
#' @param Zs Cell type specific regions of dimension R x K
#' @param a0init Default NULL. Initial mean methylation 
#' level in unmethylated regions
#' @param a1init Default NULL. Initial mean methylation
#'  level in methylated regions
#' @param sig0init Default NULL. Initial var methylation 
#' level in unmethylated regions
#' @param sig1init Default NULL. Initial var methylation 
#' level in methylated regions
#' @param tauinit Default NULL. Initial var for measurement error
#' 
#' @return A list of MLE estimates to be used in \code{estimatecc()}. 
#
#' 
#' @importFrom quadprog solve.QP
#' 
.initializeMLEs <- function(init_param_method, n, K, Ys, Zs, a0init, a1init,
                            sig0init, sig1init, tauinit)
{
  
  init_theta <- .initialize_theta(n=n, K=K, alpha0=a0init,
                                  alpha1=a1init, sig0=sig0init,
                                  sig1=sig1init, tau=tauinit)
  
  # Initial guess for cell composition estimates
  x0 = (1-Zs) * init_theta$alpha0 + (Zs) * init_theta$alpha1
  init_pi_mle <- abs(t(apply(Ys, 2, function(ys){
    solve.QP(Dmat = (t(x0)%*%x0),
             dvec = t(x0)%*%t(t(ys)), 
             Amat = cbind(rep(1,K), diag(K)),
             bvec = c(1, rep(0, K)), 
             meq = 1)$sol })))
  
  
  return(list("init_pi_mle" = init_pi_mle, "init_theta" = init_theta))
}

#' @title Helper function to take the product of Z and
#' cell composition estimates
#'
#' @description Helper function which is the product of
#' Z and pi_mle
#' 
#' @return A list of output after taking the product of Z 
#' and cell composition mle estimates to be used in 
#' \code{.methylcc_estep()}.
#'
#' @param Zs Cell type specific regions of dimension R x K
#' @param pi_mle cell composition MLE estimates
#' 
.WFun <- function(Zs, pi_mle){
  W0 = (1-Zs) %*% t(pi_mle)
  W1 = Zs %*% t(pi_mle)
  list("W0" = W0, "W1" = W1)
}

#' @title Expectation step
#'
#' @description Expectation step in EM algorithm for methylCC
#'
#' @param Ys observed methylation levels in samples provided by user
#' of dimension R x n
#' @param Zs Cell type specific regions of dimension R x K
#' @param current_pi_mle cell composition MLE estimates of dimension K x n
#' @param current_theta other parameter estimates in EM algorithm
#' @param meth_status Indicator function corresponding to regions that
#' are unmethylated (meth_status=0) or methylated (meth_status=1)
#' 
#' @return List of expected value of the first two moments 
#' of the random effects (or the E-Step in the EM algorithm)
#' used in \code{.methylcc_engine()}
#'
.methylcc_estep <- function(Ys, Zs, current_pi_mle, 
                             current_theta,
                             meth_status = 0)
{
  R <- nrow(Zs)
  n <- ncol(Ys)
  W <- .WFun(Zs=Zs, pi_mle=current_pi_mle)
  
  # dim(mat11.inv) = (R*n x R*n) in block diagonal form
  mat11.inv <- lapply(seq_len(nrow(Zs)), function(r){
    # take inverse of individual blocks
    solve( t(t(W$W0[r,])) %*% t(W$W0[r,]) * current_theta$sig0 +
             t(t(W$W1[r,])) %*% t(W$W1[r,]) * current_theta$sig1 +
             diag(current_theta$tau, n) )    })
  
  if(meth_status == 0){
    # dim(mat12.0) = (R*n x R) in block diagonal form
    mat12.0 <- lapply(seq_len(nrow(Zs)), function(r){
      t(t(W$W0[r,])) * current_theta$sig0 })
    
    # dim(mat22.0) = (R x R)
    mat22.0 <- diag(current_theta$sig0, R)
    
    # dim(mom1) = dim(mom2) = (R x 1)
    fast.mat.mom1 <- t(t(vapply(seq_len(nrow(Zs)), function(r){
      t(mat12.0[[r]]) %*% mat11.inv[[r]] %*%
        (t(t(Ys[r,])) - current_theta$alpha0*t(t(W$W0[r,])) - 
           current_theta$alpha1*t(t(W$W1[r,])) )
    }, FUN.VALUE = numeric(1))))
    fast.mat.mom2 <- diag(vapply(seq_len(nrow(Zs)), function(r){
      t(mat12.0[[r]]) %*% mat11.inv[[r]] %*% mat12.0[[r]]
    }, FUN.VALUE = numeric(1)), R)
    
    mom1 = current_theta$alpha0 * matrix(rep(1, R), ncol=1) + fast.mat.mom1
    mom2 = mom1^2 + t(t(diag(mat22.0 - fast.mat.mom2 )))
  }
  if(meth_status == 1){
    # dim(mat12.1) = (R*n x R) in block diagonal form
    mat12.1 <- lapply(seq_len(nrow(Zs)), function(r){
      t(t(W$W1[r,])) * current_theta$sig1 })
    
    # dim(mat22.1) = (R x R)
    mat22.1 <- diag(current_theta$sig1, R)
    
    # dim(mom1) = dim(mom2) = (R x 1)
    fast.mat.mom1 <- t(t(vapply(seq_len(nrow(Zs)), function(r){
      t(mat12.1[[r]]) %*% mat11.inv[[r]] %*%
        (t(t(Ys[r,])) - current_theta$alpha0*t(t(W$W0[r,])) - 
           current_theta$alpha1*t(t(W$W1[r,])) )
    }, FUN.VALUE = numeric(1))))
    fast.mat.mom2 <- diag(vapply(seq_len(nrow(Zs)), function(r){
      t(mat12.1[[r]]) %*% mat11.inv[[r]] %*% mat12.1[[r]]
    }, FUN.VALUE = numeric(1)), R)
    
    mom1 = current_theta$alpha1 * matrix(rep(1, R), ncol=1) + fast.mat.mom1
    mom2 = mom1^2 + t(t(diag(mat22.1 - fast.mat.mom2 )))
  }
  list("mom1" = mom1, "mom2" = mom2)
}

#' @title Maximization step
#'
#' @description Maximization step in EM Algorithm for methylCC
#'
#' @param Ys observed methylation levels in samples provided by user
#' of dimension R x n
#' @param Zs Cell type specific regions of dimension R x K
#' @param current_pi_mle cell composition MLE estimates of dimension K x n
#' @param current_theta other parameter estimates in EM algorithm
#' @param estep0 Results from expectation step for unmethylated regions
#' @param estep1 Results from expectation step for methylated regions
#'
#' @return A list of the updated MLEs (or the M-Step 
#' in the EM algorithm) used in \code{.methylcc_engine()}
#'
#' @importFrom quadprog solve.QP
#' 
.methylcc_mstep <- function(Ys, Zs, current_pi_mle, 
                             current_theta, 
                             estep0, estep1)
{
  R <- dim(Zs)[1]
  K <- dim(Zs)[2]
  n <- ncol(Ys)
  new_theta <- data.frame("alpha0" = mean(estep0$mom1),
                          "alpha1" = mean(estep1$mom1),
                          "sig0" = mean(estep0$mom2) - mean(estep0$mom1)^2,
                          "sig1" = mean(estep1$mom2) - mean(estep1$mom1)^2)
  x0 <- (sweep((1-Zs), 1, estep0$mom1, FUN = "*") +
           sweep(Zs,     1, estep1$mom1, FUN = "*"))
  new_pi_mle <- abs(t(apply(Ys, 2, function(ys){
    solve.QP(Dmat = (t(x0)%*%x0),
             dvec = t(x0)%*%t(t(ys)), Amat = cbind(rep(1,K), diag(K)),
             bvec = c(1, rep(0, K)), meq = 1)$sol })))
  tau_est <- (1/(R*n)) * sum((Ys - x0 %*% t(new_pi_mle))^2)
  new_theta <- cbind(new_theta, "tau" = tau_est,
                     "lik_fun" = -((R*n)/2)*log(2*pi*tau_est) -
                       (1/(2*tau_est)) * sum((Ys - x0 %*% t(new_pi_mle))^2) -
                       ((R*n)/2) * log(2*pi*new_theta$sig0) -
                       (n/(2*new_theta$sig0)) * 
                              sum((estep0$mom1 - new_theta$alpha0)^2) -
                       ((R*n)/2) * log(2*pi*new_theta$sig1) -
                       (n/(2*new_theta$sig1)) *
                              sum((estep1$mom1 - new_theta$alpha1)^2))
  
  return(list("new_theta" = new_theta, "new_pi_mle" = new_pi_mle))
}


#' @title .methylcc_engine
#' 
#' @description Helper function for estimatecc
#' 
#' @param Ys observed methylation levels in samples provided by user
#' of dimension R x n
#' @param Zs Cell type specific regions of dimension R x K
#' @param current_pi_mle cell composition MLE estimates of dimension K x n
#' @param current_theta other parameter estimates in EM algorithm
#' @param epsilon Add here.
#' @param max_iter Add here.
#' 
#' @return A list of MLE estimates that is used in 
#' \code{estimatecc()}. 
#' 
.methylcc_engine <- function(Ys, Zs, current_pi_mle, 
                              current_theta, epsilon, max_iter)
{
  final_theta <- NULL
  final_pi_mle <- NULL
  count = 0
  
  repeat{
    # E-Step
    EStep0 <- 
      .methylcc_estep(Ys = Ys, Zs = Zs, 
                       current_pi_mle = current_pi_mle, 
                       current_theta = current_theta, 
                       meth_status = 0)
    EStep1 <- 
      .methylcc_estep(Ys = Ys, Zs = Zs, 
                       current_pi_mle = current_pi_mle, 
                       current_theta = current_theta, 
                       meth_status = 1)
    
    # M-Step
    update_step <- 
      .methylcc_mstep(Ys = Ys, Zs = Zs, 
                       current_pi_mle = current_pi_mle, 
                       current_theta = current_theta, 
                       estep0=EStep0,
                       estep1=EStep1)
    
    if(abs(update_step$new_theta$lik_fun - current_theta$lik_fun) >= epsilon){
      final_pi_mle <- update_step$new_pi_mle
      final_theta <- rbind(final_theta, update_step$new_theta)
      current_pi_mle <- update_step$new_pi_mle
      current_theta <- update_step$new_theta
      count = count + 1
      
      if(count > max_iter){
        break
      }
    } else {
      break
    }
  }
  return(list("pi_mle" = final_pi_mle, "theta" = final_theta))
}

#' @title Finding differentially methylated regions
#' 
#' @description This function uses the \code{FlowSorted.Blood.450k}
#' whole blood reference methylomes with six cell types 
#' to identify differentially methylated regions.  
#'
#' @param verbose TRUE/FALSE argument specifying if verbose
#' messages should be returned or not. Default is TRUE.
#' @param gr_target Default is NULL. However, the user 
#' can provide a GRanges object from the \code{object} 
#' in \code{estimatecc}. Before starting the procedure to 
#' find differentially methylated regions, the intersection
#' of the \code{gr_target} and GRanges object from the 
#' reference methylomes (\code{FlowSorted.Blood.450k}). 
#' @param include_cpgs TRUE/FALSE. Should individual CpGs
#' be returned. Default is FALSE. 
#' @param include_dmrs TRUE/FALSE. Should differentially 
#' methylated regions be returned. Default is TRUE. User
#' can turn this to FALSE and search for only CpGs. 
#' @param num_cpgs The max number of CpGs to return 
#' for each cell type. Default is 50.
#' @param num_regions The max number of DMRs to return 
#' for each cell type. Default is 50. 
#' @param bumphunter_beta_cutoff The \code{cutoff} threshold 
#' in \code{bumphunter()} in the \code{bumphunter} package. 
#' @param dmr_up_cutoff A cutoff threshold for identifying 
#' DMRs that are methylated in one cell type, but not in the 
#' other cell types. 
#' @param dmr_down_cutoff A cutoff threshold for identifying 
#' DMRs that are not methylated in one cell type, but 
#' methylated in the other cell types.
#' @param dmr_pval_cutoff  A cutoff threshold for the p-values 
#' when identifying DMRs that are methylated in one cell 
#' type, but not in the other cell types (or vice versa). 
#' @param cpg_pval_cutoff A cutoff threshold for the p-values 
#' when identifying differentially methylated CpGs that are
#' methylated in one cell type, but not in the other cell
#' types (or vice versa). 
#' @param cpg_up_dm_cutoff A cutoff threshold for identifying 
#' differentially methylated CpGs that are methylated in 
#' one cell type, but not in the other cell types. 
#' @param cpg_down_dm_cutoff A cutoff threshold for identifying 
#' differentially methylated CpGs that are not methylated in 
#' one cell type, but are methylated in the other cell types.  
#' @param pairwise_comparison TRUE/FAlSE of whether all pairwise
#' comparisons (e.g. methylated in Granulocytes and Monocytes, 
#' but not methylated in other cell types). Default if FALSE. 
#' @param mset_train_flow_sort Default is NULL. However, a user
#' can provide a \code{MethylSet} object after processing the 
#' \code{FlowSorted.Blood.450k} dataset. The default normalization
#' is \code{preprocessIllumina()}. 
#' 
#' @return A list of data frames and GRanges objects.
#' 
#' @import minfi 
#' @import GenomicRanges
#' @import FlowSorted.Blood.450k
#' @importFrom Biobase pData
#' @importFrom bumphunter clusterMaker loessByCluster bumphunter 
#' @importFrom genefilter rowttests
#' @importFrom plyranges arrange 
#' @importFrom S4Vectors queryHits
#' @importFrom stats model.matrix
#' @importFrom utils head
#' 
.find_dmrs <- function(verbose=TRUE, gr_target=NULL,
                      include_cpgs = FALSE, include_dmrs = TRUE,
                      num_cpgs=50, num_regions=50, 
                      bumphunter_beta_cutoff = 0.2, 
                      dmr_up_cutoff = 0.5, dmr_down_cutoff = 0.4,
                      dmr_pval_cutoff = 1e-11, cpg_pval_cutoff = 1e-08,
                      cpg_up_dm_cutoff = 0, cpg_down_dm_cutoff = 0, 
                      pairwise_comparison = FALSE, 
                      mset_train_flow_sort=NULL) {

  if(is.null(mset_train_flow_sort)){
    FlowSorted.Blood.450k <- updateObject(FlowSorted.Blood.450k)
    mset_train_flow_sort <- preprocessIllumina(FlowSorted.Blood.450k)
    mset_train_flow_sort <- mapToGenome(mset_train_flow_sort, 
                                        mergeManifest = FALSE)
    rm(FlowSorted.Blood.450k)
  }
  
  # create training object to identify DMRs
  IDs = c("Gran", "CD4T", "CD8T", "Bcell","Mono", "NK")
  mset_train_flow_sort <- mset_train_flow_sort[, 
              (pData(mset_train_flow_sort)$CellType %in% IDs) ]
  
  # remove outliers
  mset_train_flow_sort <- mset_train_flow_sort[, 
               pData(mset_train_flow_sort)$Sample_Name != "CD8+_105"] 
  
  # find celltype specific regions using only overlap CpGs in target object
  if(!is.null(gr_target)){ 
    # which of the 450K CpGs overlap with the target CpGs
    zz <- findOverlaps(granges(mset_train_flow_sort), gr_target) 
    mset_train_flow_sort <- mset_train_flow_sort[queryHits(zz), ]
    if(verbose){
      mes <- "[estimatecc] gr_target is not null. Using %s overlapping CpGs."
      message(sprintf(mes, nrow(mset_train_flow_sort)))
    }
  }  
  
  # extract beta values, phenotypic information and GRanges objects
  pd <- as.data.frame(pData(mset_train_flow_sort))
  gr <- granges(mset_train_flow_sort)
  p_beta <- getBeta(mset_train_flow_sort, type = "Illumina") # beta values
  colnames(p_beta) = pd$Sample_Name = rownames(pd) = 
                     gsub("\\+","", pd$Sample_Name)
  cell <- factor(pd$CellType, levels = IDs)
  cell_levels <- levels(cell)
  
  # extract chromosome and position information for each probe in 
  #   450k array (need this for regions)
  chr <- as.character(seqnames(gr))
  pos <- start(gr)
  cl <- clusterMaker(chr, pos) # Create clusters using clusterMaker()
  
  # define design matrix to search for DMRs
  xmat = cbind(rep(1, length(cell)), model.matrix(~cell - 1))
  colnames(xmat) = c("Intercept", cell_levels)
  
  if(pairwise_comparison){
    all_poss = as.matrix(expand.grid(c(0,1), c(0,1), c(0,1), 
                                     c(0,1), c(0,1), c(0,1)))
    # remove the cases containing all methylated or unmethylated. 
    all_poss = all_poss[2:32,] 
    all_poss <- (all_poss == TRUE)
    colnames(all_poss) <- cell_levels
  } else { 
    all_poss = diag(length(cell_levels))
    all_poss <- (all_poss == TRUE)
    colnames(all_poss) <- cell_levels
  }
  
  regions_all <- GRanges() 
  zmat <- c() # regions_all, will contain all celltype-specific DMRs
  for(ind in seq_len(nrow(all_poss))){
    if(verbose){
      if(include_dmrs & include_cpgs){
        mes <- "[estimatecc] Searching for %s cell type-specific 
                regions and CpGs."
        message(sprintf(mes, paste(cell_levels[all_poss[ind,]], collapse=",")))
      } 
      if(include_dmrs & !include_cpgs) {
        mes <- "[estimatecc] Searching for %s cell type-specific regions."
        message(sprintf(mes, paste(cell_levels[all_poss[ind,]], collapse=",")))
      }
      if(!include_dmrs & include_cpgs) {
        mes <- "[estimatecc] Searching for %s cell type-specific CpGs"
        message(sprintf(mes, paste(cell_levels[all_poss[ind,]], collapse=",")))
      }
    }
    
    x_ind = cbind("Intercept" = xmat[, "Intercept"],
                  "cellTypes" = rowSums(
                    as.matrix(xmat[, cell_levels[all_poss[ind,]] ],
                              ncols = length(cell_levels[all_poss[ind,]]))))
    
    if(!include_dmrs){ 
      gr_regions_up <- GRanges()
      gr_regions_down <- GRanges()
    }
    
    if(include_dmrs){
      bumps = bumphunter(object = p_beta, design = x_ind, 
                         chr = chr, pos = pos, cluster = cl, 
                         cutoff = bumphunter_beta_cutoff, 
                         B = 0, smooth = FALSE, 
                         smoothFunction = loessByCluster)
      
      # y_regions are the beta values collapsed (CpGs averaged) by regions 
      # from bumphunter
      y_regions <- t(apply(bumps$table[,7:8], 1, function(z){
        colMeans(p_beta[(z[1]):(z[2]),,drop=FALSE]) } ))
      
      tmp <- rowttests(y_regions,factor(x_ind[,"cellTypes"]))
      bumps$table$p.value <- tmp$p.value
      bumps$table$dm <- tmp$dm 
      bumps$table$dmr_up_max_diff <- 
        apply(abs(sweep(y_regions, 2, x_ind[,"cellTypes"], FUN = "-")), 1, max)
      bumps$table$dmr_down_max_diff <- 
        apply(abs(sweep(y_regions, 2, (1 - x_ind[,"cellTypes"]), FUN = "-")), 
              1, max)
      
      # # Only include region with more than 1 CpG (L > 1)
      # #       OR only 1 CpG in region if no other larger regions possible
      L = dm <- NULL 
      keep_ind_regions <- (bumps$table$L > 1 | 
                             (bumps$table$L==1 & bumps$table$clusterL == 1)) & 
        (bumps$table$p.value < dmr_pval_cutoff)  # ideally less than 1e-11
      
      bump_mat_up <- bumps$table[keep_ind_regions & bumps$table$dm < 0 &
                            # ideally less than 0.6
                            bumps$table$dmr_up_max_diff<dmr_up_cutoff,] 
      bump_mat_up <- bump_mat_up[order(-bump_mat_up$L, bump_mat_up$dm), ]
      if(nrow(bump_mat_up) > 0){
        gr_regions_up <- makeGRangesFromDataFrame(bump_mat_up, 
                                                  keep.extra.columns=TRUE)
        mcols(gr_regions_up)$dmr_status <- rep("DMR", length(gr_regions_up))
        gr_regions_up <- gr_regions_up[, names(mcols(gr_regions_up)) %in% 
                              c("indexStart", "indexEnd", "L", "dm", "p.value", 
                                "dmr_status", "dmr_up_max_diff")]
        names(mcols(gr_regions_up))[
          names(mcols(gr_regions_up)) == "dmr_up_max_diff"] <- "dmr_max_diff"

        gr_regions_up <- gr_regions_up %>% arrange(-L, dm) %>% 
                              head(num_regions)
      } else {
        gr_regions_up <- GRanges()
      }
      
      bump_mat_down <- bumps$table[(keep_ind_regions) & bumps$table$dm > 0 & 
                                     bumps$table$dmr_down_max_diff < 
                                     dmr_down_cutoff,] # ideally less than 0.8
      bump_mat_down <- bump_mat_down[order(-bump_mat_down$L, 
                                           -bump_mat_down$dm), ]
      if(nrow(bump_mat_down) > 0){
        gr_regions_down <- makeGRangesFromDataFrame(bump_mat_down, 
                                                    keep.extra.columns=TRUE)
        mcols(gr_regions_down)$dmr_status <- 
                        rep("DMR", length(gr_regions_down))
        gr_regions_down <- gr_regions_down[, names(mcols(gr_regions_down)) %in%
                            c("indexStart", "indexEnd", "L", "dm", "p.value", 
                              "dmr_status", "dmr_down_max_diff")]
        names(mcols(gr_regions_down))[names(mcols(gr_regions_down)) 
                                      == "dmr_down_max_diff"] <- "dmr_max_diff"
        gr_regions_down <- gr_regions_down %>% 
                              arrange(-L, -dm) %>% 
                              head(num_regions)
      } else {
        gr_regions_down <- GRanges()
      }
    }  
    
    if(include_cpgs){
      tstats <- rowttests(p_beta, factor(x_ind[,"cellTypes"]))
      tstats <- tstats[(tstats[, "p.value"] < cpg_pval_cutoff),] 
      
      tstats_up <- tstats[order(tstats[, "dm"], decreasing = FALSE), ]
      # at a min should be less than 0
      tstats_up <- tstats_up[tstats_up$dm < cpg_up_dm_cutoff,] 
        
      probe_keep <- rownames(tstats_up)[seq_len(min(nrow(tstats_up), 
                                                    num_cpgs))]
      if(length(probe_keep) > 0){
        gr_probe <- granges(mset_train_flow_sort[probe_keep,])
        mcols(gr_probe) <- tstats[probe_keep, c("dm", "p.value")]
        mcols(gr_probe)$L <- rep(1, length(probe_keep))
        mcols(gr_probe)$indexStart <- match(probe_keep,
                                            rownames(mset_train_flow_sort))
        mcols(gr_probe)$indexEnd <- match(probe_keep, 
                                          rownames(mset_train_flow_sort))
        mcols(gr_probe)$dmr_status <- rep("CpG", length(gr_probe))
        gr_regions_up <- unique(c(gr_regions_up, 
                                  gr_probe[,c("indexStart", "indexEnd", 
                                              "L", "p.value", "dm", 
                                              "dmr_status")]))
        gr_regions_up <- gr_regions_up %>% arrange(-L, dm) %>% 
                            head(num_regions)
      } 
      
      tstats_down <- tstats[order(tstats[, "dm"], decreasing = TRUE), ]
      # at a min should be greater than 0
      tstats_down <- tstats_down[tstats_down$dm > cpg_down_dm_cutoff,] 
      probe_keep <- rownames(tstats_down)[seq_len(min(nrow(tstats_down), 
                                                      num_cpgs))]
      if(length(probe_keep) > 0){
        gr_probe <- granges(mset_train_flow_sort[probe_keep,])
        mcols(gr_probe) <- tstats[probe_keep, c("dm", "p.value")]
        mcols(gr_probe)$L <- rep(1, length(probe_keep))
        mcols(gr_probe)$indexStart <- match(probe_keep,
                                            rownames(mset_train_flow_sort))
        mcols(gr_probe)$indexEnd <- match(probe_keep, 
                                          rownames(mset_train_flow_sort))
        mcols(gr_probe)$dmr_status <- rep("CpG", length(gr_probe))
        gr_regions_down <- unique(c(gr_regions_down, 
                                    gr_probe[,c("indexStart", "indexEnd", 
                                                "L", "p.value", "dm", 
                                                "dmr_status")]))
        gr_regions_down <- gr_regions_down %>% 
                            arrange(-L, -dm) %>% 
                            head(num_regions)
      } 
    }
    
    mcols(gr_regions_up)$status <- rep("Up", length(gr_regions_up))
    mcols(gr_regions_down)$status <- rep("Down", length(gr_regions_down))
    bump_mat_all <- c(gr_regions_up, gr_regions_down)
    mcols(bump_mat_all)$cellType <- rep(paste(cell_levels[all_poss[ind,]], 
                                              collapse=","), 
                                        length(bump_mat_all)) 
    
    if(verbose){
      if(include_dmrs & include_cpgs){
        mes <- "[estimatecc] Found %s %s cell type-specific regions and CpGs."
        message(sprintf(mes, length(bump_mat_all), 
                        paste(cell_levels[all_poss[ind,]], collapse=",")))
      } 
      if(include_dmrs & !include_cpgs) {
        mes <- "[estimatecc] Found %s %s cell type-specific regions."
        message(sprintf(mes, length(bump_mat_all), 
                        paste(cell_levels[all_poss[ind,]], collapse=",")))
      }
      if(!include_dmrs & include_cpgs) {
        mes <- "[estimatecc] Found %s %s cell type-specific CpGs."
        message(sprintf(mes, length(bump_mat_all),
                        paste(cell_levels[all_poss[ind,]], collapse=",")))
      }
    }
    
    if(length(bump_mat_all) > 0){ 
      regions_all <- c(regions_all, bump_mat_all)
    }
    if(length(gr_regions_up) > 0){
      zmat <- rbind(zmat, t(replicate(min(length(gr_regions_up), num_regions), 
                                      as.numeric(all_poss[ind,]))))
    }
    if(length(gr_regions_down) > 0){
      zmat <- rbind(zmat, t(replicate(min(length(gr_regions_down), 
                                          num_regions), 
                                      as.numeric(!all_poss[ind,]))))
    }
  }
  colnames(zmat) <- cell_levels
  
  y_regions <- t(apply(
          as.data.frame(mcols(regions_all))[,seq_len(2)],1,function(ind){
    colMeans(p_beta[(ind[1]):(ind[2]),,drop=FALSE])
  }))
  
  profiles <- vapply(.splitit(cell), 
                     FUN = function(ind){ rowMeans(y_regions[,ind])}, 
                     FUN.VALUE = numeric(nrow(y_regions)))

  removeMe <- duplicated(regions_all)
  list(regions_all = regions_all[!removeMe,], 
       zmat = zmat[!removeMe,], 
       y_regions = y_regions[!removeMe,], 
       profiles = profiles[!removeMe,],
       cell = cell, cell_mat = all_poss, 
       cell_levels = cell_levels, pd = pd)
} 

#' @name FlowSorted.Blood.450k.sub 
#' @title A reduced size of the FlowSorted.Blood.450k dataset 
#' 
#' @description A reduced size of the 
#' \code{FlowSorted.Blood.450k} dataset 
#' 
#' The object was created using the script in /inst and
#' located in the /data folder.
#' 
#' @docType data 
#' @format A RGset object with 2e5 rows (probes)
#' and 6 columns (whole blood samples).
#' 
#' @rdname FlowSorted.Blood.450k.sub
#' 
NULL 

#' @name onMethRegions
#' @title Methylated regions for all celltypes
#'
#' @description
#' This is the script used to create the
#' \code{onMethRegions} data set.
#' The purpose is use in the \code{estimate_cc()}
#' function.
#'
#' The object was created using the script in /inst and
#' located in the /data folder.
#'
#' @docType data
#' @format add more here.
#'
#' @rdname onMethRegions
#'
NULL

#' @name offMethRegions
#' @title Unmethylated regions for all celltypes
#'
#' @description
#' This is the script used to create the
#' \code{offMethRegions} data set.
#' The purpose is use in the \code{estimate_cc()}
#' function.
#'
#' The object was created using the script in /inst and
#' located in the /data folder.
#'
#' @docType data
#' @format add more here.
#'
#' @rdname offMethRegions
#'
NULL

#' @title Accessors for the 'cell_counts' slot of a estimatecc object.
#' 
#' @description Accessors for the 'cell_counts' slot of a estimatecc object.
#' 
#' @usage
#' \S4method{cell_counts}{estimatecc}(object)
#'
#' @docType methods
#' @name cell_counts
#' @rdname cell_counts
#' @aliases cell_counts cell_counts,estimatecc-method
#' @param object an object of class \code{estimatecc}.
#' 
#' @return Returns the cell composition estimates
#' 
#' @export
#' 
#' @examples
#' # This is a reduced version of the FlowSorted.Blood.450k 
#' # dataset available by using BiocManager::install("FlowSorted.Blood.450k),
#' # but for purposes of the example, we use the smaller version 
#' # and we set \code{demo=TRUE}. For any case outside of this example for 
#' # the package, you should set \code{demo=FALSE} (the default). 
#' 
#' dir <- system.file("data", package="methylCC")
#' files <- file.path(dir, "FlowSorted.Blood.450k.sub.RData") 
#' if(file.exists(files)){
#'     load(file = files)
#' 
#'     set.seed(12345)
#'     est <- estimatecc(object = FlowSorted.Blood.450k.sub, demo = TRUE) 
#'     cell_counts(est)
#'  }   
#' 
setMethod(
  f = "cell_counts", 
  signature = "estimatecc",
  definition = function(object) {
    return(object@cell_counts)
  }
)

## ----echo=FALSE, results="hide", message=FALSE--------------------------------
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)

## ----style, echo=FALSE, results='asis'----------------------------------------
BiocStyle::markdown()

## ----loadlibs, message=FALSE, warning=FALSE-----------------------------------
library(FlowSorted.Blood.450k)
library(methylCC)
library(minfi)
library(tidyr)
library(dplyr)
library(ggplot2)

## ----data-load, message=FALSE-------------------------------------------------
# Phenotypic information about samples
head(pData(FlowSorted.Blood.450k))

# RGChannelSet
rgset <- FlowSorted.Blood.450k[,
                pData(FlowSorted.Blood.450k)$CellTypeLong %in% "Whole blood"]

## ----run-estimatecc1, message=FALSE-------------------------------------------
set.seed(12345)
est <- estimatecc(object = rgset) 
est

## ----run-estimatecc-summaries-------------------------------------------------
cell_counts(est)

## ----run-minfi-estimateCellCounts---------------------------------------------
sampleNames(rgset) <- paste0("Sample", 1:6)

est_minfi <- minfi::estimateCellCounts(rgset)
est_minfi

## ----compare-estimates--------------------------------------------------------
df_minfi = gather(cbind("samples" = rownames(cell_counts(est)),
                        as.data.frame(est_minfi)),
                  celltype, est, -samples)

df_methylCC = gather(cbind("samples" = rownames(cell_counts(est)),
                           cell_counts(est)),
                     celltype, est, -samples)

dfcombined <- full_join(df_minfi, df_methylCC, 
                               by = c("samples", "celltype"))

ggplot(dfcombined, aes(x=est.x, y = est.y, color = celltype)) +
    geom_point() + xlim(0,1) + ylim(0,1) +
    geom_abline(intercept = 0, slope = 1) +
    xlab("Using minfi::estimateCellCounts()") + 
    ylab("Using methylCC::estimatecc()") +
    labs(title = "Comparing cell composition estimates")

## ----sessionInfo, results='markup'--------------------------------------------
sessionInfo()

#' @title the estimatecc class
#'
#' @description Objects of this class store all 
#' the values needed information to work with a
#' estimatecc object
#' 
#' @slot summary information about the samples and 
#' regions used to estimate cell composition
#' @slot cell_counts cell composition estimates
#' 
#' @return \code{summary} returns the summary information 
#' about the cell composition estimate procedure and 
#' \code{cell_counts} returns the cell composition estimates
#' 
#' @name estimatecc-class
#' @import methods
#' @exportClass estimatecc
#' @aliases estimatecc-class
#'
#' @examples
#' # This is a reduced version of the FlowSorted.Blood.450k 
#' # dataset available by using BiocManager::install("FlowSorted.Blood.450k),
#' # but for purposes of the example, we use the smaller version 
#' # and we set \code{demo=TRUE}. For any case outside of this example for 
#' # the package, you should set \code{demo=FALSE} (the default). 
#' 
#' dir <- system.file("data", package="methylCC")
#' files <- file.path(dir, "FlowSorted.Blood.450k.sub.RData") 
#' if(file.exists(files)){
#'     load(file = files)
#' 
#'     set.seed(12345)
#'     est <- estimatecc(object = FlowSorted.Blood.450k.sub, demo = TRUE) 
#'     cell_counts(est)
#'  }   
#' 
setClass(
    Class = "estimatecc", 
    slot = list(
        summary = "list",
        cell_counts = "data.frame")
)

setMethod("show", "estimatecc",
          function(object){
            cat("estimatecc: Estimate Cell Composition of Whole Blood 
                Samples using DNA methylation\n")
            cat("   Input object class: ", object@summary$class, "\n")
            cat("   Reference cell types: ", object@summary$celltypes, "\n")
            cat("   Number of Whole Blood Samples: ", 
                paste(object@summary$n_samples), "\n")
            cat("   Name of Whole Blood Samples: ", 
                paste(object@summary$sample_names), "\n")
          }
)

#' @title A reduced size of the FlowSorted.Blood.450k dataset 
#' 
#' @description A reduced size of the FlowSorted.Blood.450k dataset 
#' 
#' @docType data 
#' @format A RGset object with 2e5 rows (probes)
#' and 6 columns (whole blood samples).
#' 
library(FlowSorted.Blood.450k)
data(FlowSorted.Blood.450k)
# take a random sample to make object size in build smaller
set.seed(12345)
cpg_ids <- sample(seq_len(nrow(FlowSorted.Blood.450k)), 1.5e5)
FlowSorted.Blood.450k.sub <- FlowSorted.Blood.450k[cpg_ids,
            pData(FlowSorted.Blood.450k)$CellTypeLong %in% "Whole blood"]

## Save FlowSorted.Blood.450k.sub
save(FlowSorted.Blood.450k.sub, 
     file = "data/FlowSorted.Blood.450k.sub.RData", 
     compress='xz') 

#' @title Regions with low methylation values in six pure blood cell types
#'
#' @description
#' This script is used to create a ADD ME
#'
#' @docType data
#'
library(FlowSorted.Blood.450k)
library(minfi)
data(FlowSorted.Blood.450k)

Mset <- preprocessIllumina(updateObject(FlowSorted.Blood.450k))
Mset <- mapToGenome(Mset, mergeManifest = FALSE)

# Only consider autosomes
rowKeep <- which(!seqnames(granges(Mset)) %in% c("chrX","chrY"))
Mset <- Mset[rowKeep,]

# Based on MDS plots, we see sample 13 is an oulier; we remove it
IDs = c("CD8T","CD4T", "NK","Bcell","Mono","Gran")
colKeep <- which(pData(Mset)$CellType %in% IDs)
colKeep = setdiff(colKeep, 13)

# order samples by cell types (IDs)
colKeep = colKeep[order(factor(pData(Mset[, colKeep])$CellType, levels = IDs))]
Mset <- Mset[, colKeep]

pd = as.data.frame(pData(Mset))
pd$CellType <- factor(pd$CellType, levels = IDs)
p = getBeta(Mset, type = "Illumina")    # beta values
colnames(p) = pd$Sample_Name = rownames(pd) = gsub("\\+","",pd$Sample_Name)

## Create Genomic Ranges object
gr_train450k <- granges(Mset)
chr <- as.character(seqnames(gr_train450k))
pos <- start(gr_train450k)


## Calculate posterior probablity of being in group 1 (groups: 0, 1)
##      using Mclust and find d0 and d1 regions using p0 data
library(mclust)
probZ1 = sapply(seq_len(ncol(p)), function(x){
   out = Mclust(p[,x], G = 2)
   out$z[,2]
})
row.names(probZ1) <- NULL
colnames(probZ1) <- colnames(p)

Z0dat = regionFinder(1 / rowMax(probZ1), chr = chr, pos = pos,
                     cluster = cl, cutoff = 1 / 0.05 )
offMethRegions = Z0dat[Z0dat$L > 3,] # DMR regions with at least 3 probes

## Save offMethRegions
save(offMethRegions, file = "data/offMethRegions.RData", compress='xz') 


#' @title Regions with low methylation values in six pure blood cell types
#'
#' @description
#' This script is used to create a ADD ME
#'
#' @docType data
#' @format A MethylSet object with 1e4 rows (probes)
#' and 58 columns (samples).
#' 
Z1dat = regionFinder(1 / (rowMax(1-probZ1)+1e-10), chr = chr, pos = pos,
                     cluster = cl, cutoff = 1 / 0.05)
onMethRegions = Z1dat[Z1dat$L>3,] # DMR regions with at least 3 probes

## Save onMethRegions
save(onMethRegions, file = "data/onMethRegions.RData", compress='xz') 

#' @title Estimate cell composition from DNAm data
#'
#' @description Estimate cell composition from DNAm data
#'
#' @param object an object can be a \code{RGChannelSet}, 
#' \code{GenomicMethylSet} or \code{BSseq} object
#' @param find_dmrs_object If the user would like to supply 
#' different differentially methylated regions, they can 
#' use the output from the \code{find_dmrs} function 
#' to supply different regions to \code{estimatecc}. 
#' @param verbose TRUE/FALSE argument specifying if verbose
#' messages should be returned or not. Default is TRUE.
#' @param epsilon Threshold for EM algorithm to check
#' for convergence. Default is 0.01.
#' @param max_iter Maximum number of iterations for EM
#' algorithm. Default is 100 iterations.
#' @param take_intersection TRUE/FALSE asking if only the CpGs 
#' included in \code{object} should be used to find DMRs. 
#' Default is FALSE.
#' @param include_cpgs TRUE/FALSE. Should individual CpGs
#' be returned. Default is FALSE. 
#' @param include_dmrs TRUE/FALSE. Should differentially 
#' methylated regions be returned. Default is TRUE. 
#' @param init_param_method method to initialize parameter estimates.
#' Choose between "random" (randomly sample) or "known_regions"
#' (uses unmethyalted and methylated regions that were identified
#' based on Reinus et al. (2012) cell sorted data.).
#' Defaults to "random".
#' @param a0init Default NULL. Initial mean 
#' methylation level in unmethylated regions
#' @param a1init Default NULL. Initial mean 
#' methylation level in methylated regions
#' @param sig0init Default NULL. Initial var 
#' methylation level in unmethylated regions
#' @param sig1init Default NULL. Initial var 
#' methylation level in methylated regions
#' @param tauinit Default NULL. Initial var 
#' for measurement error
#' @param demo TRUE/FALSE. Should the function 
#' be used in demo mode to shorten examples in 
#' package. Defaults to FALSE. 
#' 
#' @return A object of the class \code{estimatecc} that 
#' contains information about the cell composition 
#' estimation (in the \code{summary} slot) and 
#' the cell composition estimates themselves 
#' (in the \code{cell_counts} slot).
#' 
#' @aliases estimatecc
#' 
#' @docType methods
#' @name estimatecc
#' @importFrom quadprog solve.QP
#'
#' @export
#' 
#' @examples
#' # This is a reduced version of the FlowSorted.Blood.450k 
#' # dataset available by using BiocManager::install("FlowSorted.Blood.450k),
#' # but for purposes of the example, we use the smaller version 
#' # and we set \code{demo=TRUE}. For any case outside of this example for 
#' # the package, you should set \code{demo=FALSE} (the default). 
#' 
#' dir <- system.file("data", package="methylCC")
#' files <- file.path(dir, "FlowSorted.Blood.450k.sub.RData") 
#' if(file.exists(files)){
#'     load(file = files)
#' 
#'     set.seed(12345)
#'     est <- estimatecc(object = FlowSorted.Blood.450k.sub, demo = TRUE) 
#'     cell_counts(est)
#'  }   
#' 
estimatecc <- function(object, find_dmrs_object = NULL, verbose = TRUE, 
                       epsilon = 0.01, max_iter = 100, 
                       take_intersection = FALSE,
                       include_cpgs = FALSE, include_dmrs = TRUE,
                       init_param_method = "random", a0init = NULL,
                       a1init = NULL, sig0init = NULL, sig1init = NULL, 
                       tauinit = NULL, demo = FALSE)
{

if(!demo){
  if(!(is(object, "RGChannelSet") || is(object, "GenomicMethylSet") || 
       is(object, "BSseq"))){
  stop("The object must be a RGChannelSet, GenomicMethylSet or BSseq object'.")
  }
  
  if(!(init_param_method %in% c("random", "known_regions")) ){
    stop("The init_param_method must be set to 'random' or 'known_regions'.")
  }
  
  if(is.null(find_dmrs_object)){
    if(!take_intersection){
      dmrs_found <- suppressWarnings(.find_dmrs(verbose = verbose, 
                                                include_cpgs = include_cpgs, 
                                                include_dmrs = include_dmrs))
    } else {
      eout <- .extract_raw_data(object)
      dmrs_found <- suppressWarnings(.find_dmrs(verbose = verbose, 
                                                gr_target=eout$gr_object, 
                                                include_cpgs = include_cpgs, 
                                                include_dmrs = include_dmrs))
      rm(eout)
    }    
      celltype_specific_dmrs <- granges(dmrs_found$regions_all)
      mcols(celltype_specific_dmrs) <- dmrs_found$zmat
  }
  
  if(!is.null(find_dmrs_object)){
      if(verbose){
        message("[estimatecc] Using regions in the find_dmrs_object argument.")
      }
      celltype_specific_dmrs <- granges(find_dmrs_object$regions_all)
      mcols(celltype_specific_dmrs) <- find_dmrs_object$zmat
  }
  
  dat <- .preprocess_estimatecc(object, verbose=verbose,
                      init_param_method=init_param_method, 
                      celltype_specific_dmrs = celltype_specific_dmrs)
  ymat <- as.matrix(dat$ymat)
  n <- dat$n
  ids <- dat$ids
  zmat <- dat$zmat
  K = ncol(zmat)
  R = nrow(zmat)
  
  if(init_param_method == "known_regions"){
    a0init = dat$a0init; sig0init = dat$sig0init
    a1init = dat$a1init; sig1init = dat$sig1init
  }
  
  # set up objects
  cell_counts  <- data.frame(array(NA, dim=c(n,K)))
  nregions_final = array(NA, dim = n)                
  samples_with_na <- apply(ymat, 2, function(x) { any(is.na(x)) })
  
  ## Include verbose messages about parameter estimation
  if(verbose){
    mes <- "[estimatecc] Starting parameter estimation using %s regions."
    message(sprintf(mes, R))
  }
  
  if(any(samples_with_na)){
    
    # samples with NAs in rows         
    for(ii in which(samples_with_na)){
      keep_rows <- c(!is.na(ymat[,ii]))
      ymat_sub <- t(t(ymat[keep_rows,ii]))
      zmat_sub <- zmat[keep_rows,]
      nregions_final[ii] <- sum(keep_rows)
      
      # Initialize MLEs
      init_step <- 
        .initializeMLEs(init_param_method = init_param_method,
                        n = n, K = K, 
                        Ys = ymat_sub, Zs = zmat_sub, 
                        a0init = a0init, a1init = a1init, 
                        sig0init = sig0init, sig1init = sig1init, 
                        tauinit = tauinit)
      
      # Run EM algorithm
      finalMLEs <- 
        .methylcc_engine(Ys = ymat_sub, Zs = zmat_sub,
                          current_pi_mle = init_step$init_pi_mle,
                          current_theta = init_step$init_theta,
                          epsilon=epsilon, max_iter = max_iter)
      
      cell_counts[ii,] <- as.data.frame(finalMLEs$pi_mle)
     }
  } 
  
  if(any(!samples_with_na)){
    
    ymat_sub <- as.matrix(ymat[, !samples_with_na])
    cut_samples <- factor(cut(seq_len(ncol(ymat_sub)),
                        breaks = unique(c(seq(0, ncol(ymat_sub), by = 100), 
                                          ncol(ymat_sub)))))
    
    final_mles <- NULL
    for(ind in seq_len(length(levels(cut_samples)))){
      keep_inds <- (cut_samples == levels(cut_samples)[ind])
      
      init_step <- try(
        .initializeMLEs(init_param_method = init_param_method,
                        n = n, K = K, 
                        Ys = as.matrix(ymat_sub[, keep_inds]), Zs = zmat, 
                        a0init = a0init, a1init = a1init, 
                        sig0init = sig0init, sig1init = sig1init, 
                        tauinit = tauinit), silent = TRUE)
      if(is(init_step, "try-error")){
    message(      
  "[estimatecc] There are not a sufficient number of differentially 
  methylated regions (DMRs) for cell composition estimation. Try 
  including both differntially methylated CpGs and DMRs by modifying 
  the estimatecc(include_cpgs=TRUE, include_dmrs=TRUE) function.")
  stop("Exiting the estimation now.")
      }

      # Run EM algorithm
      finalMLEs <- 
        .methylcc_engine(Ys = as.matrix(ymat_sub[, keep_inds]), Zs = zmat,
                         current_pi_mle = init_step$init_pi_mle,
                         current_theta = init_step$init_theta,
                         epsilon=epsilon, max_iter = max_iter)
      final_mles <- rbind(final_mles, finalMLEs$pi_mle)
    }
    
    # recored results
    cell_counts[!samples_with_na,] <- as.data.frame(final_mles)
    nregions_final[!samples_with_na] <- rep(R, sum(!samples_with_na))
  }
  
  if(verbose){
    mes <- "[estimatecc] Parameter estimation complete."
    message(sprintf(mes))
  }
  
  
  results <- new("estimatecc")
  colnames(cell_counts) <- ids
  rownames(cell_counts) <- colnames(ymat)
  results@cell_counts <- cell_counts
  results@summary <- list("class" = class(object),
                          "n_samples" = n, "celltypes" = ids,
                          "sample_names" = colnames(ymat),
                          "init_param_method" = init_param_method,
                          "n_regions" = nregions_final)
}

  if(demo){
    cell_counts <- 
    data.frame("Gran" = c(0.4941867, 0.5509522, 0.6014232, 0.5587378, 
                          0.7749587, 0.6190246), 
               "CD4T" = c(1.912074e-01, 1.853314e-01, 1.103946e-01,
                          1.131525e-01, 2.405420e-19, 1.777451e-01), 
               "CD8T" = c(1.827254e-01, 3.658930e-02, 1.596001e-01, 
                          9.149783e-02, 7.128668e-19, 1.104661e-01), 
               "Bcell" = c(0.06434181, 0.06984018, 0.03566914, 0.08211265, 
                           0.01367197, 0.05127689), 
               "Mono" = c(0.06753869, 0.11898253, 0.06945726, 0.05730704, 
                          0.07340088, 0.04148729), 
               "NK" = c(0.00000000, 0.03830442, 0.02345569, 0.09719214, 
                        0.13796849, 0.00000000))
    
    results <- new("estimatecc")
    ids <- c("Gran", "CD4T", "CD8T", "Bcell", "Mono", "NK")
    nam <- c("WB_105", "WB_218", "WB_261", "WB_043", "WB_160", "WB_149")
    results@cell_counts <- cell_counts
    colnames(results@cell_counts) <- ids
    rownames(results@cell_counts) <- nam
    results@summary <- list("class" = "RGChannelSet",
                            "n_samples" = 6, "celltypes" = ids,
                            "sample_names" = nam, 
                            "init_param_method" = "random",
                            "n_regions" = rep(84, 6))
  }
  return(results)
}

test_that("checking estimatecc output (without .find_dmrs)", {
  # This is a reduced version of the FlowSorted.Blood.450k 
  # dataset available by using BiocManager::install("FlowSorted.Blood.450k),
  # but for purposes of the example, we use the smaller version. 
  
  dir <- system.file("data", package="methylCC")
  files <- file.path(dir, "FlowSorted.Blood.450k.sub.RData") 
  if(file.exists(files)){
      load(file = files)
      
      set.seed(12345)
      est <- estimatecc(object = FlowSorted.Blood.450k.sub) 
      cell_counts(est)
      expect_equal(cell_counts(est)[1,1], 0.49418, tolerance = 0.0001)
  }   
 
})

#' Generic function that returns the cell composition 
#' estimates
#'
#' Given a estimatecc object, this function returns the 
#' cell composition estimates 
#' @rdname cell_counts
setGeneric(
    name = "cell_counts", 
    def = function(object) { 
        standardGeneric("cell_counts") 
    }
)

---
title: "The methylCC user's guide"
author:
- name: Stephanie C. Hicks
  affiliation: Johns Hopkins Bloomberg School of Public Health
- name: Rafael A. Irizarry 
  affiliation: Dana-Farber Cancer Institute
output:
  BiocStyle::html_document:
    toc_float: true
package: methylCC
abstract: |
 A tool to estimate the cell composition of DNA 
   methylation whole blood sample measured on any 
   platform technology (microarray and sequencing).
vignette: |
  %\VignetteIndexEntry{The methylCC user's guide}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r, echo=FALSE, results="hide", message=FALSE}
require(knitr)
opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

```{r style, echo=FALSE, results='asis'}
BiocStyle::markdown()
```


# Introduction

There are several approaches available to adjust for differents in the
relative proportion of cell types in whole blood measured from
DNA methylation (DNAm). For example, *reference-based approaches*
require the use of reference data sets made up of purified cell types to
identify cell type-specific DNAm signatures. These cell type-specific DNAm
signatures are used to estimate the relative proportions of cell types
directly, but these reference data sets are laborious and expensive to
collect. Furthermore, these reference data sets will need to be
continuously collected over time as new platform technologies emerge
measuring DNAm because the observed methylation levels for the same CpGs
in the same sample vary depending the platform technology.

In contrast, there are *reference-free approaches*,
which are based on methods related to surrogate variable analysis or
linear mixed models. These approaches do not provide
estimates of the relative proportions of cell types, but rather these
methods just remove the variability induced from the differences in
relative cell type proportions in whole blood samples.

Here, we present a statistical model that estimates the cell composition
of whole blood samples measured from DNAm. The method can be applied
to microarray or sequencing data (for example whole-genome bisulfite
sequencing data, WGBS, reduced representation bisulfite sequencing
data, RRBS). Our method is based on the
idea of identifying informative genomic regions that are clearly
methylated or unmethylated for each cell type, which permits
estimation in multiple platform technologies as cell types preserve
their methylation state in regions independent of platform despite
observed measurements being platform dependent.


# Getting Started

Load the `methylCC` R package and other packages that we'll need 
later on. 

```{r loadlibs, message=FALSE, warning=FALSE}
library(FlowSorted.Blood.450k)
library(methylCC)
library(minfi)
library(tidyr)
library(dplyr)
library(ggplot2)
```


# Data

## Whole Blood Illumina 450k Microarray Data Example

```{r data-load, message=FALSE}
# Phenotypic information about samples
head(pData(FlowSorted.Blood.450k))

# RGChannelSet
rgset <- FlowSorted.Blood.450k[,
                pData(FlowSorted.Blood.450k)$CellTypeLong %in% "Whole blood"]
```


# Using the `estimatecc()` function

## Input for `estimatecc()`
The `estimatecc()` function must have
one object as input:

1. an `object` such as an `RGChannelSet` from
the R package `minfi` or a `BSseq` object
from the R package `bsseq`. This object should 
contain observed DNAm levels at CpGs (rows) 
in a set of $N$ whole blood samples (columns). 

## Running `estimatecc()`

In this example, we are interested in estimating the cell
composition of the whole blood samples listed in the
`FlowSorted.Blood.450k` R/Bioconductor package.
To run the `methylcC::estimatecc()` function,
just provide the `RGChannelSet`. This will
create an `estimatecc` object. We
will call the object `est`.


```{r run-estimatecc1, message=FALSE}
set.seed(12345)
est <- estimatecc(object = rgset) 
est
```

To see the cell composition estimates, use the
`cell_counts()` function.

```{r run-estimatecc-summaries}
cell_counts(est)
```


## Compare to `minfi::estimateCellCounts()` 

We can also use the `estimateCellCounts()` from R/Bioconductor package
[`minfi`](http://bioconductor.org/packages/release/bioc/html/minfi.html)
to estimate the cell composition for each of the whole blood samples. 

```{r run-minfi-estimateCellCounts}
sampleNames(rgset) <- paste0("Sample", 1:6)

est_minfi <- minfi::estimateCellCounts(rgset)
est_minfi
```

Then, we can compare the estimates to `methylCC::estimatecc()`. 

```{r compare-estimates}
df_minfi = gather(cbind("samples" = rownames(cell_counts(est)),
                        as.data.frame(est_minfi)),
                  celltype, est, -samples)

df_methylCC = gather(cbind("samples" = rownames(cell_counts(est)),
                           cell_counts(est)),
                     celltype, est, -samples)

dfcombined <- full_join(df_minfi, df_methylCC, 
                               by = c("samples", "celltype"))

ggplot(dfcombined, aes(x=est.x, y = est.y, color = celltype)) +
    geom_point() + xlim(0,1) + ylim(0,1) +
    geom_abline(intercept = 0, slope = 1) +
    xlab("Using minfi::estimateCellCounts()") + 
    ylab("Using methylCC::estimatecc()") +
    labs(title = "Comparing cell composition estimates")
```

We see the estimates closely match for the six cell types. 


# SessionInfo

```{r sessionInfo, results='markup'}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllClasses.R
\docType{class}
\name{estimatecc-class}
\alias{estimatecc-class}
\title{the estimatecc class}
\value{
\code{summary} returns the summary information 
about the cell composition estimate procedure and 
\code{cell_counts} returns the cell composition estimates
}
\description{
Objects of this class store all 
the values needed information to work with a
estimatecc object
}
\section{Slots}{

\describe{
\item{\code{summary}}{information about the samples and 
regions used to estimate cell composition}

\item{\code{cell_counts}}{cell composition estimates}
}}

\examples{
# This is a reduced version of the FlowSorted.Blood.450k 
# dataset available by using BiocManager::install("FlowSorted.Blood.450k),
# but for purposes of the example, we use the smaller version 
# and we set \\code{demo=TRUE}. For any case outside of this example for 
# the package, you should set \\code{demo=FALSE} (the default). 

dir <- system.file("data", package="methylCC")
files <- file.path(dir, "FlowSorted.Blood.450k.sub.RData") 
if(file.exists(files)){
    load(file = files)

    set.seed(12345)
    est <- estimatecc(object = FlowSorted.Blood.450k.sub, demo = TRUE) 
    cell_counts(est)
 }   

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.extract_raw_data}
\alias{.extract_raw_data}
\title{Extract raw data}
\usage{
.extract_raw_data(object)
}
\arguments{
\item{object}{an object can be a \code{RGChannelSet}, 
\code{GenomicMethylSet} or \code{BSseq} object}
}
\value{
A list preprocessed objects from the 
\code{RGChannelSet}, \code{GenomicMethylSet} or 
\code{BSseq} objects to be used in \code{.preprocess_estimatecc()}.
}
\description{
Extract the methylation values and 
GRanges objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.methylcc_mstep}
\alias{.methylcc_mstep}
\title{Maximization step}
\usage{
.methylcc_mstep(Ys, Zs, current_pi_mle, current_theta, estep0, estep1)
}
\arguments{
\item{Ys}{observed methylation levels in samples provided by user
of dimension R x n}

\item{Zs}{Cell type specific regions of dimension R x K}

\item{current_pi_mle}{cell composition MLE estimates of dimension K x n}

\item{current_theta}{other parameter estimates in EM algorithm}

\item{estep0}{Results from expectation step for unmethylated regions}

\item{estep1}{Results from expectation step for methylated regions}
}
\value{
A list of the updated MLEs (or the M-Step 
in the EM algorithm) used in \code{.methylcc_engine()}
}
\description{
Maximization step in EM Algorithm for methylCC
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.pick_target_positions}
\alias{.pick_target_positions}
\title{Pick target positions}
\usage{
.pick_target_positions(target_granges, target_object = NULL,
  target_cvg = NULL, dmp_regions)
}
\arguments{
\item{target_granges}{add more here.}

\item{target_object}{an optional argument which 
contains the meta-data for \code{target_granges}. If
\code{target_granges} already contains the meta-data, do not 
need to supply \code{target_object}.}

\item{target_cvg}{coverage reads for the target object}

\item{dmp_regions}{differentially methylated regions}
}
\value{
A list of GRanges objects to be used in 
\code{.preprocess_estimatecc()}
}
\description{
Pick probes from target data using 
the indices in \code{dmp_regions}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.WFun}
\alias{.WFun}
\title{Helper function to take the product of Z and
cell composition estimates}
\usage{
.WFun(Zs, pi_mle)
}
\arguments{
\item{Zs}{Cell type specific regions of dimension R x K}

\item{pi_mle}{cell composition MLE estimates}
}
\value{
A list of output after taking the product of Z 
and cell composition mle estimates to be used in 
\code{.methylcc_estep()}.
}
\description{
Helper function which is the product of
Z and pi_mle
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{FlowSorted.Blood.450k.sub}
\alias{FlowSorted.Blood.450k.sub}
\title{A reduced size of the FlowSorted.Blood.450k dataset}
\format{A RGset object with 2e5 rows (probes)
and 6 columns (whole blood samples).}
\description{
A reduced size of the 
\code{FlowSorted.Blood.450k} dataset 

The object was created using the script in /inst and
located in the /data folder.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/estimatecc.R
\docType{methods}
\name{estimatecc}
\alias{estimatecc}
\title{Estimate cell composition from DNAm data}
\usage{
estimatecc(object, find_dmrs_object = NULL, verbose = TRUE,
  epsilon = 0.01, max_iter = 100, take_intersection = FALSE,
  include_cpgs = FALSE, include_dmrs = TRUE,
  init_param_method = "random", a0init = NULL, a1init = NULL,
  sig0init = NULL, sig1init = NULL, tauinit = NULL, demo = FALSE)
}
\arguments{
\item{object}{an object can be a \code{RGChannelSet}, 
\code{GenomicMethylSet} or \code{BSseq} object}

\item{find_dmrs_object}{If the user would like to supply 
different differentially methylated regions, they can 
use the output from the \code{find_dmrs} function 
to supply different regions to \code{estimatecc}.}

\item{verbose}{TRUE/FALSE argument specifying if verbose
messages should be returned or not. Default is TRUE.}

\item{epsilon}{Threshold for EM algorithm to check
for convergence. Default is 0.01.}

\item{max_iter}{Maximum number of iterations for EM
algorithm. Default is 100 iterations.}

\item{take_intersection}{TRUE/FALSE asking if only the CpGs 
included in \code{object} should be used to find DMRs. 
Default is FALSE.}

\item{include_cpgs}{TRUE/FALSE. Should individual CpGs
be returned. Default is FALSE.}

\item{include_dmrs}{TRUE/FALSE. Should differentially 
methylated regions be returned. Default is TRUE.}

\item{init_param_method}{method to initialize parameter estimates.
Choose between "random" (randomly sample) or "known_regions"
(uses unmethyalted and methylated regions that were identified
based on Reinus et al. (2012) cell sorted data.).
Defaults to "random".}

\item{a0init}{Default NULL. Initial mean 
methylation level in unmethylated regions}

\item{a1init}{Default NULL. Initial mean 
methylation level in methylated regions}

\item{sig0init}{Default NULL. Initial var 
methylation level in unmethylated regions}

\item{sig1init}{Default NULL. Initial var 
methylation level in methylated regions}

\item{tauinit}{Default NULL. Initial var 
for measurement error}

\item{demo}{TRUE/FALSE. Should the function 
be used in demo mode to shorten examples in 
package. Defaults to FALSE.}
}
\value{
A object of the class \code{estimatecc} that 
contains information about the cell composition 
estimation (in the \code{summary} slot) and 
the cell composition estimates themselves 
(in the \code{cell_counts} slot).
}
\description{
Estimate cell composition from DNAm data
}
\examples{
# This is a reduced version of the FlowSorted.Blood.450k 
# dataset available by using BiocManager::install("FlowSorted.Blood.450k),
# but for purposes of the example, we use the smaller version 
# and we set \\code{demo=TRUE}. For any case outside of this example for 
# the package, you should set \\code{demo=FALSE} (the default). 

dir <- system.file("data", package="methylCC")
files <- file.path(dir, "FlowSorted.Blood.450k.sub.RData") 
if(file.exists(files)){
    load(file = files)

    set.seed(12345)
    est <- estimatecc(object = FlowSorted.Blood.450k.sub, demo = TRUE) 
    cell_counts(est)
 }   

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.preprocess_estimatecc}
\alias{.preprocess_estimatecc}
\title{.preprocess_estimatecc}
\usage{
.preprocess_estimatecc(object, verbose = TRUE,
  init_param_method = "random",
  celltype_specific_dmrs = celltype_specific_dmrs)
}
\arguments{
\item{object}{an object can be a \code{RGChannelSet}, 
\code{GenomicMethylSet} or \code{BSseq} object}

\item{verbose}{TRUE/FALSE argument specifying if verbose
messages should be returned or not. Default is TRUE.}

\item{init_param_method}{method to initialize parameter estimates.
Choose between "random" (randomly sample) or "known_regions"
(uses unmethyalted and methylated regions that were identified
based on Reinus et al. (2012) cell sorted data.).
Defaults to "random".}

\item{celltype_specific_dmrs}{cell type specific differentially 
methylated regions (DMRs).}
}
\value{
A list of object to be used in estimatecc
}
\description{
This function preprocesses the data 
before the \code{estimatecc()} function
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.splitit}
\alias{.splitit}
\title{.splitit}
\usage{
.splitit(x)
}
\arguments{
\item{x}{a vector}
}
\value{
A list to be used in \code{find_dmrs()}
}
\description{
helper function to split along a variable
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{offMethRegions}
\alias{offMethRegions}
\title{Unmethylated regions for all celltypes}
\format{add more here.}
\description{
This is the script used to create the
\code{offMethRegions} data set.
The purpose is use in the \code{estimate_cc()}
function.

The object was created using the script in /inst and
located in the /data folder.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R, R/methods.R
\docType{methods}
\name{cell_counts}
\alias{cell_counts}
\alias{cell_counts,estimatecc-method}
\title{Generic function that returns the cell composition 
estimates}
\usage{
cell_counts(object)

\S4method{cell_counts}{estimatecc}(object)
}
\arguments{
\item{object}{an object of class \code{estimatecc}.}
}
\value{
Returns the cell composition estimates
}
\description{
Given a estimatecc object, this function returns the 
cell composition estimates

Accessors for the 'cell_counts' slot of a estimatecc object.
}
\examples{
# This is a reduced version of the FlowSorted.Blood.450k 
# dataset available by using BiocManager::install("FlowSorted.Blood.450k),
# but for purposes of the example, we use the smaller version 
# and we set \\code{demo=TRUE}. For any case outside of this example for 
# the package, you should set \\code{demo=FALSE} (the default). 

dir <- system.file("data", package="methylCC")
files <- file.path(dir, "FlowSorted.Blood.450k.sub.RData") 
if(file.exists(files)){
    load(file = files)

    set.seed(12345)
    est <- estimatecc(object = FlowSorted.Blood.450k.sub, demo = TRUE) 
    cell_counts(est)
 }   

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.methylcc_estep}
\alias{.methylcc_estep}
\title{Expectation step}
\usage{
.methylcc_estep(Ys, Zs, current_pi_mle, current_theta, meth_status = 0)
}
\arguments{
\item{Ys}{observed methylation levels in samples provided by user
of dimension R x n}

\item{Zs}{Cell type specific regions of dimension R x K}

\item{current_pi_mle}{cell composition MLE estimates of dimension K x n}

\item{current_theta}{other parameter estimates in EM algorithm}

\item{meth_status}{Indicator function corresponding to regions that
are unmethylated (meth_status=0) or methylated (meth_status=1)}
}
\value{
List of expected value of the first two moments 
of the random effects (or the E-Step in the EM algorithm)
used in \code{.methylcc_engine()}
}
\description{
Expectation step in EM algorithm for methylCC
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.methylcc_engine}
\alias{.methylcc_engine}
\title{.methylcc_engine}
\usage{
.methylcc_engine(Ys, Zs, current_pi_mle, current_theta, epsilon, max_iter)
}
\arguments{
\item{Ys}{observed methylation levels in samples provided by user
of dimension R x n}

\item{Zs}{Cell type specific regions of dimension R x K}

\item{current_pi_mle}{cell composition MLE estimates of dimension K x n}

\item{current_theta}{other parameter estimates in EM algorithm}

\item{epsilon}{Add here.}

\item{max_iter}{Add here.}
}
\value{
A list of MLE estimates that is used in 
\code{estimatecc()}.
}
\description{
Helper function for estimatecc
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_dmrs.R
\name{.find_dmrs}
\alias{.find_dmrs}
\title{Finding differentially methylated regions}
\usage{
.find_dmrs(verbose = TRUE, gr_target = NULL, include_cpgs = FALSE,
  include_dmrs = TRUE, num_cpgs = 50, num_regions = 50,
  bumphunter_beta_cutoff = 0.2, dmr_up_cutoff = 0.5,
  dmr_down_cutoff = 0.4, dmr_pval_cutoff = 1e-11,
  cpg_pval_cutoff = 1e-08, cpg_up_dm_cutoff = 0,
  cpg_down_dm_cutoff = 0, pairwise_comparison = FALSE,
  mset_train_flow_sort = NULL)
}
\arguments{
\item{verbose}{TRUE/FALSE argument specifying if verbose
messages should be returned or not. Default is TRUE.}

\item{gr_target}{Default is NULL. However, the user 
can provide a GRanges object from the \code{object} 
in \code{estimatecc}. Before starting the procedure to 
find differentially methylated regions, the intersection
of the \code{gr_target} and GRanges object from the 
reference methylomes (\code{FlowSorted.Blood.450k}).}

\item{include_cpgs}{TRUE/FALSE. Should individual CpGs
be returned. Default is FALSE.}

\item{include_dmrs}{TRUE/FALSE. Should differentially 
methylated regions be returned. Default is TRUE. User
can turn this to FALSE and search for only CpGs.}

\item{num_cpgs}{The max number of CpGs to return 
for each cell type. Default is 50.}

\item{num_regions}{The max number of DMRs to return 
for each cell type. Default is 50.}

\item{bumphunter_beta_cutoff}{The \code{cutoff} threshold 
in \code{bumphunter()} in the \code{bumphunter} package.}

\item{dmr_up_cutoff}{A cutoff threshold for identifying 
DMRs that are methylated in one cell type, but not in the 
other cell types.}

\item{dmr_down_cutoff}{A cutoff threshold for identifying 
DMRs that are not methylated in one cell type, but 
methylated in the other cell types.}

\item{dmr_pval_cutoff}{A cutoff threshold for the p-values 
when identifying DMRs that are methylated in one cell 
type, but not in the other cell types (or vice versa).}

\item{cpg_pval_cutoff}{A cutoff threshold for the p-values 
when identifying differentially methylated CpGs that are
methylated in one cell type, but not in the other cell
types (or vice versa).}

\item{cpg_up_dm_cutoff}{A cutoff threshold for identifying 
differentially methylated CpGs that are methylated in 
one cell type, but not in the other cell types.}

\item{cpg_down_dm_cutoff}{A cutoff threshold for identifying 
differentially methylated CpGs that are not methylated in 
one cell type, but are methylated in the other cell types.}

\item{pairwise_comparison}{TRUE/FAlSE of whether all pairwise
comparisons (e.g. methylated in Granulocytes and Monocytes, 
but not methylated in other cell types). Default if FALSE.}

\item{mset_train_flow_sort}{Default is NULL. However, a user
can provide a \code{MethylSet} object after processing the 
\code{FlowSorted.Blood.450k} dataset. The default normalization
is \code{preprocessIllumina()}.}
}
\value{
A list of data frames and GRanges objects.
}
\description{
This function uses the \code{FlowSorted.Blood.450k}
whole blood reference methylomes with six cell types 
to identify differentially methylated regions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.initializeMLEs}
\alias{.initializeMLEs}
\title{.initializeMLEs}
\usage{
.initializeMLEs(init_param_method, n, K, Ys, Zs, a0init, a1init, sig0init,
  sig1init, tauinit)
}
\arguments{
\item{init_param_method}{method to initialize parameter estimates.
Choose between "random" (randomly sample) or "known_regions"
(uses unmethyalted and methylated regions that were identified
based on Reinus et al. (2012) cell sorted data.).
Defaults to "random".}

\item{n}{Number of samples}

\item{K}{Number of cell types}

\item{Ys}{observed methylation levels in samples 
provided by user of dimension R x n}

\item{Zs}{Cell type specific regions of dimension R x K}

\item{a0init}{Default NULL. Initial mean methylation 
level in unmethylated regions}

\item{a1init}{Default NULL. Initial mean methylation
level in methylated regions}

\item{sig0init}{Default NULL. Initial var methylation 
level in unmethylated regions}

\item{sig1init}{Default NULL. Initial var methylation 
level in methylated regions}

\item{tauinit}{Default NULL. Initial var for measurement error}
}
\value{
A list of MLE estimates to be used in \code{estimatecc()}.
}
\description{
Helper functions to initialize 
MLEs in \code{estimatecc()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper_functions.R
\name{.initialize_theta}
\alias{.initialize_theta}
\title{.initialize_theta}
\usage{
.initialize_theta(n, K, alpha0 = NULL, alpha1 = NULL, sig0 = NULL,
  sig1 = NULL, tau = NULL)
}
\arguments{
\item{n}{Number of samples}

\item{K}{Number of cell types}

\item{alpha0}{Default NULL. Initial mean methylation 
level in unmethylated regions}

\item{alpha1}{Default NULL. Initial mean methylation
level in methylated regions}

\item{sig0}{Default NULL. Initial var methylation 
level in unmethylated regions}

\item{sig1}{Default NULL. Initial var methylation 
level in methylated regions}

\item{tau}{Default NULL. Initial var for measurement error}
}
\value{
A data frame with initial parameter estimates
to be used in \code{.initializeMLEs()}.
}
\description{
Creates a container with initial theta
parameter estimates
}
